import httpx
import yaml
import json
import requests
from typing import List, Optional, Dict, Any, Union
from ..dtos.permission import PermissionDetailedDto,PermissionDetailedFilterDto

class ExternalService:
    def __init__(self, config_file: str):
        self.config = self.load_config(config_file)
        self._policyv2_url = self.config.get("PolicyServerUrlv2", {})

    def load_config(self, config_file: Union[str, Dict[str, Any]]) -> Dict[str, Any]:
        if config_file is Dict[str, Any]: return config_file
        with open(config_file, 'r') as f:
            return yaml.safe_load(f)
        
    async def execute_in_policy_api_get_roles(self, filter: PermissionDetailedFilterDto, token: str) -> Optional[List[PermissionDetailedDto]]:
        headers = {'Accept': 'application/json'}

        url = f"{self._policyv2_url}policies/endpoint-item-permissions-with-policies/{filter.user_id}"
        json_data = json.dumps(filter.__dict__)
        if token:
            headers["Authorization"] = f"Bearer {token}"

        async with httpx.AsyncClient() as http_client:
            response = await http_client.get(url, headers=headers, params=json_data)

            if response.is_success:
                resp_obj = response.json().get("Data", [])
                return [PermissionDetailedDto.from_dict(item) for item in resp_obj]

            return None



from fastapi import FastAPI,Request
import jwt
from ..enums.header_names import HeaderNames
from ..dtos.user import UserClaims
from ..dtos.request import CarbonBaseRequest, CarbonRoute


BearerTokenClaimMapper = {
    "sub": "ClientId",
    "tenant-id": "TenantId",
    "god-user": "GodUser"
}

async def BearerTokenMiddleware(request: Request, call_next):
    authorization = request.headers.get(HeaderNames.Authorization)

    if authorization:
        bearer_token = authorization.split(" ")[1] if HeaderNames.Bearer in authorization else None
        
        if bearer_token:
            claims = jwt.decode(bearer_token, options={"verify_signature": False})

            # Başlıkları düzenle
            if claims:

                request.state.token = bearer_token
                # "GodUser" başlığını kaldır
                if "GodUser" in request.headers:
                    del request.headers["GodUser"]
                
                # "TenantId" başlığını kaldır
                if claims.get("god-user") != "true":
                    if "TenantId" in request.headers:
                        del request.headers["TenantId"]

                # "ClientId" başlığını kaldır
                if "ClientId" in request.headers:
                    del request.headers["ClientId"]

                # Claim'leri başlıklara ekle
                for claim_type, mapped_key in BearerTokenClaimMapper.items():
                    if claim_type in claims:
                        request.headers.__dict__[mapped_key] = claims[claim_type]
                user = UserClaims(claims=claims)
                request.state.user_claims = user

    response = await call_next(request)
    return response


import requests
import yaml
from datetime import datetime, timedelta
from typing import Optional, Dict, Any, Union

class AuthenticationType:
    BasicAuth = 0
    OAuth2 = 1
    OneM2MSpecific = 2
    OneM2MConnector = 3
    OAuthClientCredentials = 4

class AuthenticationInfo:
    def __init__(self, primary_access_id: str, primary_access_id_expire_date: Optional[datetime], name: str):
        self.auth_type = None
        self.primary_access_id = primary_access_id
        self.primary_access_id_expire_date = primary_access_id_expire_date
        self.name = name

class BasicAuth(AuthenticationInfo):
    def __init__(self, access_token: str, access_token_expiry_date: Optional[datetime], name: str):
        super().__init__(access_token, access_token_expiry_date, name)
        self.access_token = access_token
        self.access_token_expiry_date = access_token_expiry_date
        self.auth_type = AuthenticationType.BasicAuth

class OneM2MAuth(AuthenticationInfo):
    def __init__(self, application_entity_id: str, application_entity_id_expiry_date: Optional[datetime], name: str):
        super().__init__(application_entity_id, application_entity_id_expiry_date, name)
        self.application_entity_id = application_entity_id
        self.application_entity_id_expiry_date = application_entity_id_expiry_date
        self.auth_type = AuthenticationType.OneM2MSpecific

class AuthHttpClientFactory:
    def __init__(self, config_file: Union[str, Dict[str, Any]]):
        self.config = self.load_config(config_file)
        self.authentications = {}
        self.current_session_headers = {}

    def load_config(self, config_file: Union[str, Dict[str, Any]]) -> Dict[str, Any]:
        if config_file is Dict[str, Any]: return config_file
        with open(config_file, 'r') as f:
            return yaml.safe_load(f)

    def drop_authentication(self, name: str) -> bool:
        if name in self.authentications:
            del self.authentications[name]
            return True
        return False

    def re_authenticate(self, name: str) -> Optional[AuthenticationInfo]:
        self.drop_authentication(name)
        return self.create_authentication(name)

    def create_authentication(self, name: str, reauth: bool = False) -> Optional[AuthenticationInfo]:
        if not reauth:
            if name in self.authentications and self.authentications[name].primary_access_id_expire_date > datetime.now():
                return self.authentications[name]

        related_section = self.config.get("Authorizations", {}).get(name)
        if not related_section or not related_section.get("AuthenticationType"):
            return None

        auth_type = int(related_section["AuthenticationType"])

        if auth_type == AuthenticationType.BasicAuth:
            credentials = {
                "username": related_section["UserName"],
                "password": related_section["Password"],
                "scope": related_section["Scope"],
                "client_id": related_section["ClientId"],
                "client_secret": related_section["Secret"]
            }
            token_response = requests.post(
                self.config["JwtSettings"]["Authority"] + "/connect/token",
                data={
                    "grant_type": "password",
                    "username": credentials["username"],
                    "password": credentials["password"],
                    "scope": credentials["scope"],
                    "client_id": credentials["client_id"],
                    "client_secret": credentials["client_secret"]
                }
            )

            if token_response.ok:
                data = token_response.json()
                acc_token = data["access_token"]
                basic_auth = BasicAuth(acc_token, datetime.now() + timedelta(seconds=data["expires_in"]), name)
                self.authentications[name] = basic_auth
                return basic_auth
            else:
                raise Exception("Identity server authentication error: " + str(token_response.status_code))

        elif auth_type == AuthenticationType.OAuthClientCredentials:
            credentials = {
                "client_id": related_section["ClientId"],
                "client_secret": related_section["Secret"],
                "scope": related_section["Scope"]
            }
            token_response = requests.post(
                self.config["JwtSettings"]["Authority"] + "/connect/token",
                data={
                    "grant_type": "client_credentials",
                    "client_id": credentials["client_id"],
                    "client_secret": credentials["client_secret"],
                    "scope": credentials["scope"]
                }
            )

            if token_response.ok:
                data = token_response.json()
                authentication_info = BasicAuth(data["access_token"], datetime.now() + timedelta(seconds=data["expires_in"]), name)
                self.authentications[name] = authentication_info
                return authentication_info
            else:
                raise Exception("Identity server authentication error: " + token_response.json().get('error'))

        elif auth_type == AuthenticationType.OneM2MSpecific:
            aeid = related_section["ApplicationEntityId"]
            one_m2m_auth = OneM2MAuth(aeid, datetime.max, name)
            one_m2m_auth.fqdn = related_section["FQDN"]
            self.authentications[name] = one_m2m_auth
            return one_m2m_auth

        return None

    def get_authentication(self, name: str) -> Optional[AuthenticationInfo]:
        if name in self.authentications and self.authentications[name].primary_access_id_expire_date > datetime.now():
            return self.authentications[name]
        return self.re_authenticate(name)

    def get_auth_names(self) -> list:
        return list(self.authentications.keys())

    def send_request(self, name: str, request: requests.Request) -> requests.Response:
        client = requests.Session()
        auth_info = self.get_authentication(name)

        if auth_info and auth_info.auth_type == AuthenticationType.BasicAuth:
            client.headers['Authorization'] = f'Bearer {auth_info.primary_access_id}'
        
        response = client.send(request)

        if response.status_code == 401:
            self.re_authenticate(name)
            client.headers['Authorization'] = f'Bearer {auth_info.primary_access_id}'
            response = client.send(request)

        return response


from fastapi import  Request,Response
from typing import Optional
from fastapi.routing import APIRoute
from typing import Callable, List

from ..dtos.user import UserClaims

class CarbonBaseRequest(Request):
    @property
    def user_claims(self) -> Optional[UserClaims]:
        return getattr(self.state, 'user_claims', None)

    @user_claims.setter
    def user_claims(self, value: UserClaims) -> None:
        self.state.user_claims = value

class CarbonRoute(APIRoute):
    def get_route_handler(self) -> Callable:
        original_route_handler = super().get_route_handler()

        async def custom_route_handler(request: CarbonBaseRequest) -> Response:
            request = CarbonBaseRequest(request.scope, request.receive)
            return await original_route_handler(request)

        return custom_route_handler


from typing import Optional

class UserClaims():
    def __init__(self, claims):
        self._claims = claims  # Gizli bir değişken tanımlıyoruz

    @property
    def claims(self):
        return self._claims

    @claims.setter
    def claims(self, value):
        self._claims = value  # Setter ile değeri değiştirebiliyoruz
    
    def get_user_id(self) -> Optional[str]:
        return self.claims.get('sub')

    def get_user_full_name(self) -> Optional[str]:
        return self.claims.get('fullname')

    def get_user_name(self) -> Optional[str]:
        return self.claims.get('name') or self.get_user_full_name()

    def get_tenant_id(self) -> Optional[str]:
        return self.claims.get('TenantId') or self.claims.get('tenant-id')

    def get_organization_id(self) -> Optional[str]:
        return self.claims.get('OrganizationId') or self.claims.get('organization-id')


    def check_if_god_user(self) -> bool:
        is_god_user = self.claims.get('GodUser') or self.claims.get('god-user')
        return is_god_user is not None and is_god_user.lower() == 'true'